from os.path import (
    expanduser,
    join
)

from sys import exit

import argparse

from mfa_tools.list import print_profiles
from mfa_tools.login import (
    get_aws_credentials,
    save_credentials
)


__version__ = "0.2.0"


def parse_args() -> argparse.Namespace:
    """Create argument parser.

    Returns
    -------
    argparse.Namespace
        object with arguments and values.
    """
    config_file = join(expanduser("~"), ".aws", "config")
    mfa_file = join(expanduser("~"), ".aws", "mfa_credentials")
    aws_file = join(expanduser("~"), ".aws", "credentials")

    parser = argparse.ArgumentParser(
        prog="awslogin",
        description="AWS MFA Tool",
        usage="%(prog)s [options]",
        epilog="Helping manage AWS Session tokens for MFA authentication.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument(
        "--version",
        action="version",
        version=f"%(prog)s {__version__}"
        )
    parser.add_argument(
        "--profile",
        default="default",
        help="aws profile to get mfa serial."
        )
    parser.add_argument(
        "--token",
        type=str,
        default=None,
        help="mfa token to login."
    )
    parser.add_argument(
        "--config",
        type=str,
        metavar="CONFIG_FILE",
        default=config_file,
        help="path to aws config file."
    )
    parser.add_argument(
        "--mfa",
        type=str,
        metavar="MFA_FILE",
        default=mfa_file,
        help="path to mfa credentials file."
    )
    parser.add_argument(
        "--aws",
        type=str,
        metavar="AWS_FILE",
        default=aws_file,
        help="path to aws credentials file."
    )
    parser.add_argument(
        "--export",
        action="store_true",
        default=False,
        help="show export command, does NOT update credentials file."
    )
    parser.add_argument(
        "--list",
        action="store_true",
        default=False,
        help="list all profiles on AWS folder."
    )

    return parser.parse_args()


def main():
    """Process credentials request."""
    args = parse_args()

    if args.list:
        print_profiles(args.config, args.mfa, args.aws)
        exit(0)

    credentials = get_aws_credentials(args.profile, args.config, args.mfa, args.token)

    aws_credentials = {
        "AWS_ACCESS_KEY_ID": credentials["AccessKeyId"],
        "AWS_SECRET_ACCESS_KEY": credentials["SecretAccessKey"],
        "AWS_SESSION_TOKEN": credentials["SessionToken"],
        "AWS_SESSION_TOKEN_EXPIRATION": credentials["Expiration"]
    }

    if args.export:
        for key, value in aws_credentials.items():
            print(f"export {key}={value}")
    else:
        save_credentials(args.profile, aws_credentials, args.aws)


if __name__ == "__main__":
    main()



"""Tool to get AWS credentials from AWS STS when MFA is required to AWS CLI."""
from configparser import ConfigParser
from typing import (
    Dict,
    Optional
)

from sys import (
    exit,
    stderr
)

import json
import os
import subprocess


def get_aws_credentials(
    aws_profile: str,
    aws_config_file: str,
    mfa_credentials_file: str,
    aws_mfa_token: Optional[str] = None
) -> Dict[str, str]:
    """Get AWS credentials for a given profile.

    Parameters
    ----------
    aws_profile : str
        Profile name in the AWS config/mfa files.
    aws_config_file : str
        AWS config file with MFA serial for the profile.
    mfa_credentials_file : str
        MFA credentials file to access AWS STS for the profile.
    aws_mfa_token : Optional[str], optional
        MFA token for AWS STS, by default None (it will be asked).

    Returns
    -------
    Dict[str, str]
        AWS Session credentials for the profile.
    """
    config = ConfigParser()
    config.read(aws_config_file)

    try:
        profile = config[aws_profile]
    except KeyError:
        print(f"AWS profile {aws_profile} not found at {aws_config_file}.", file=stderr)
        exit(-1)

    try:
        mfa_serial = profile["mfa_serial"]
    except KeyError:
        print(f"AWS profile {aws_profile} does not have mfa_serial configured at {aws_config_file}", file=stderr)
        exit(-1)

    mfa_token = input(f"MFA token for profile {aws_profile}: ") if aws_mfa_token is None else aws_mfa_token

    # Delete any AWS environment variable in place
    env_vars = os.environ.copy()
    for key in env_vars.keys():
        if key.startswith("AWS_"):
            del env_vars[key]

    # AWS credentials file with access keys for the profile MFA
    env_vars["AWS_SHARED_CREDENTIALS_FILE"] = mfa_credentials_file

    # Get the session token
    proc = subprocess.Popen(
        [
            "aws", "sts", "get-session-token",
            "--profile", aws_profile,
            "--serial-number", mfa_serial,
            "--token-code", mfa_token
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        env=env_vars
    )
    data, _ = proc.communicate()
    if proc.returncode != 0:
        print(f"Get session token for profile {aws_profile} failed: {data.decode('utf-8')}.", file=stderr)
        exit(-1)

    # Get temporary credentials
    credentials = json.loads(data.decode("utf-8"))["Credentials"]
    return credentials


def save_credentials(aws_profile: str, aws_credentials: Dict[str, str], credentials_file: str) -> None:
    """Save the session credentials into the AWS credential file.

    Parameters
    ----------
    aws_profile : str
        AWS profile name in the credentials file.
    aws_credentials : Dict[str, str]
        AWS Session credentials for the profile.
    credentials_file : str
        AWS credentials file location.
    """
    aws_file = ConfigParser()
    aws_file.read(credentials_file)
    aws_file[aws_profile] = aws_credentials

    with open(credentials_file, "w") as f:
        aws_file.write(f)

    print(f"AWS credentials file {credentials_file} for profile {aws_profile} updated.")



"""Module to print AWS profiles."""
from configparser import ConfigParser
from datetime import datetime

from typing import Tuple


def get_token_expiration(parser: ConfigParser, profile: str) -> Tuple[str, str]:
    """Return token expiration date and whether it is expired.

    Parameters
    ----------
    parser : ConfigParser
        Parser with all configuration files.
    profile : str
        Profile name.

    Returns
    -------
    Tuple[str, str]
        Tuple with expiration date and whether it is expired.
    """
    expiration = parser.get(profile, 'aws_session_token_expiration', fallback=None)

    if expiration is None:
        return "", ""

    dt = datetime.strptime(expiration, "%Y-%m-%dT%H:%M:%S%z")

    expired = "N" if dt > datetime.now(dt.tzinfo) else "Y"

    return f"{dt:%Y-%m-%d %H:%M:%S}", expired


def has_configured(parser: ConfigParser, profile: str, parameter: str) -> str:
    """Return whether a parameter is configured.

    Parameters
    ----------
    parser : ConfigParser
        Parser with all configuration files.
    profile : str
        Profile name.
    parameter : str
        Parameter name.

    Returns
    -------
    str
        Whether the parameter is configured.
    """
    return "N" if parser.get(profile, parameter, fallback=None) is None else "Y"


def print_profiles(config_file: str, mfa_file: str, credentials_file: str) -> None:
    """Print all profiles and data related.

    Parameters
    ----------
    config_file : str
        AWS configuration file.
    mfa_file : str
        MFA credentials file.
    credentials_file : str
        AWS credentials file.
    """
    config = ConfigParser()
    config.read(config_file)

    mfa = ConfigParser()
    mfa.read(mfa_file)

    credentials = ConfigParser()
    credentials.read(credentials_file)

    profiles = sorted(set(config.sections() + mfa.sections() + credentials.sections()))

    print("")
    print(
        f"|{'Profile name':^22}|{'Region':^11}|{'MFA':^5}|{'AccessKey':^11}" +
        f"|{'Token':^7}|{'Expiration':^21}|{'Expired':^9}|"
    )
    print(f"|{'-' * 22}|{'-' * 11}|{'-' * 5}|{'-' * 11}|{'-' * 7}|{'-' * 21}|{'-' * 9}|")

    for profile in profiles:
        region = config.get(profile, "region", fallback=None)
        has_mfa = has_configured(config, profile, "mfa_serial")
        has_key = has_configured(mfa, profile, "aws_access_key_id")
        has_token = has_configured(credentials, profile, "aws_session_token")
        expiration, expired = get_token_expiration(credentials, profile)

        print(f"| {profile:21}|{region:^11}|{has_mfa:^5}|{has_key:^11}|{has_token:^7}|{expiration:^21}|{expired:^9}|")

    print("")



from setuptools import (
    find_packages,
    setup
)


setup(
    name="aws-mfa-tools",
    version="0.2.0",
    packages=find_packages(),
    author="Daniel Ferrari",
    description="AWS MFA tools to be used on command line.",
    license="MIT",
    keywords="aws cli mfa login",
    url="https://github.com/FerrariDG/aws-mfa-tools",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: Freely Distributable",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: Microsoft",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Security",
        "Topic :: Software Development :: Build Tools"
    ],
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    entry_points={
        "console_scripts": [
            "awslogin=mfa_tools.main:main"
        ]
    },
    python_requires=">=3.7"
)
